/*
** Module   : AbstractTempTable.java
** Abstract : Abstract base class for static and dynamic temp-table 4GL objects.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 SVL 20131115 Created initial version.
** 002 CA  20131209 The dynamic flag must be passed up the c'tor call chain.
** 003 HC  20131215 Implemented ADM-DATA and UNIQUE-ID attribute.
** 004 SVL 20140106 ERROR-STRING attribute and some error handling functions are now handled by
**                  this class.
*/

package com.goldencode.p2j.persist;

import com.goldencode.p2j.util.*;

/**
 * Abstract base class for static and dynamic temp-table 4GL objects.
 */
public abstract class AbstractTempTable
extends HandleChain
implements TempTable,
           Deletable
{
   /** Corresponds UNIQUE-ID attribute. **/
   private final integer uniqueId = new integer();

   /** Corresponds ERROR-STRING attribute. **/
   private final character errorString = new character();

   /** Corresponds ADM-DATA attribute. **/
   protected final character admData = new character();

   /**
    * Default constructor.
    * 
    * @param    dynamic
    *           Flag indicating if this is a static or a dynamic temp-table.
    */
   public AbstractTempTable(boolean dynamic)
   {
      super(dynamic);
      uniqueId.assign(UniqueIdGenerator.getUniqueId(UniqueIdGenerator.IdKind.TEMP_TABLE));
   }

   /**
    * Gets the the unique ID number associated to this object by the
    * underlying system. Not the same as the handle value itself.
    *
    * @return   The integer number of the children.
    */
   @Override
   public integer getUniqueID()
   {
      return new integer(this.uniqueId);
   }

   /**
    * Sets the ERROR-STRING attribute of 4GL temp-table.
    *
    * @param   errStr
    *          The new value for the ERROR-STRING attribute.
    */
   public void changeErrorString(character errStr)
   {
      if (!prepared().getValue())
      {
         ErrorManager.displayError(12346,
               "ERROR-STRING can only be used with a PREPARED temp-table that is not a BEFORE-TABLE",
               false);
         ErrorManager.recordOrThrowError(3131,
               "Unable to set attribute ERROR-STRING in widget of type TEMP-TABLE",
               false);
         return;
      }

      this.errorString.assign(errStr);
   }

   /**
    * Obtain the current ERROR-STRING attribute of 4GL temp-table.
    *
    * @return  the current ERROR-STRING of the temp-table
    */
   public character errorString()
   {
      return errorString;
   }

   /**
    * Get the value of the ADM-DATA attribute.
    * 
    * @return   See above.
    */
   @Override
   public character getADMData()
   {
      return new character(this.admData);
   }

   /**
     * Set the value of the ADM-DATA attribute.
     * 
     * @param    value
     *           The new value.
     */
   @Override
   public void setADMData(String value)
   {
      this.admData.assign(value);
   }

   /**
    * Set the value of the ADM-DATA attribute.
    * 
    * @param    value
    *           The new value.
    */
   @Override
   public void setADMData(character value)
   {
      this.admData.assign(value);
   }

   /**
    * Display "CREATE-LIKE must be the first method on a newly CREATEd TEMP-TABLE, or a newly
    * CLEARed one" error message.
    */
   protected void displayCreateLikeNotFirst()
   {
      ErrorManager.displayError(9041,
            "CREATE-LIKE must be the first method on a newly CREATEd TEMP-TABLE, " +
                  "or a newly CLEARed one",
            false);
   }

   /**
    * Display "Definitional TEMP-TABLE methods (ADD-NEW/LIKE) may not be called after
    * TEMP-TABLE-PREPARE has been called unless a CLEAR is done first, and may never be called on
    * a DEFINED static table" error message.
    */
   protected void displayPrepared()
   {
      ErrorManager.displayError(9042,
            "Definitional TEMP-TABLE methods (ADD-NEW/LIKE) may not be called after " +
                  "TEMP-TABLE-PREPARE has been called unless a CLEAR is done first, and may never " +
                  "be called on a DEFINED static table",
            false);

   }

   /**
    * Display "TEMP-TABLE is already prepared--ignoring" error message.
    */
   protected void displayPreparedIgnoring()
   {
      ErrorManager.displayError(9043, "TEMP-TABLE is already prepared--ignoring", false);
   }

   /**
    * Display "Unable to assign UNKNOWN value to attribute UNDO on TEMP-TABLE widget" error
    * message.
    */
   protected void displayUnableAssignUnknownUndo()
   {
      ErrorManager.displayError(4083,
            "**Unable to assign UNKNOWN value to attribute UNDO on TEMP-TABLE widget",
            false);
   }
}
